/***************************************************************************
 *   Copyright (C) 2008 by peter komar   *
 *   peter@linux-nlyt   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "dwnmainimpl.h"
#include "newdlgimpl.h"
#include "ui_authenticationdialog.h"

#include <QtGui>
#include <QtNetwork>

DwnMainImpl::DwnMainImpl(QWidget *parent)
   : QWidget(parent)
{
	setupUi(this);
	
	QStringList labels;
	labels << tr("Link") << tr("Progress") << tr("Directory:"); 
	listDownload->setHeaderLabels(labels);
	
	http = new QHttp(this);

	init_conections();
	stopBtn->setEnabled(false);
}


DwnMainImpl::~DwnMainImpl()
{
}

/*!
    \fn DwnMainImpl::init_conections()
 */
void DwnMainImpl::init_conections()
{
	connect(addBtn, SIGNAL(clicked()),this,SLOT(slot_new_download()));
	connect(removeBtn, SIGNAL(clicked()),this,SLOT(remove_item()));
	connect(startBtn, SIGNAL(clicked()),this,SLOT(start_download()));
	connect(stopBtn, SIGNAL(clicked()),this,SLOT(stop_download()));
	connect(aboutBtn, SIGNAL(clicked()),this,SLOT(slot_about()));
	connect(closeBtn, SIGNAL(clicked()),this,SLOT(slot_exit()));
	connect(http, SIGNAL(requestFinished(int, bool)),this, SLOT(httpRequestFinished(int, bool)));
	connect(http, SIGNAL(dataReadProgress(int, int)),this, SLOT(updateDataReadProgress(int, int)));
	connect(http, SIGNAL(responseHeaderReceived(const QHttpResponseHeader &)),this, SLOT(readResponseHeader(const QHttpResponseHeader &)));
	connect(http, SIGNAL(authenticationRequired(const QString &, quint16, QAuthenticator *)),
		this, SLOT(slotAuthenticationRequired(const QString &, quint16, QAuthenticator *)));
}

/*!
    \fn DwnMainImpl::stop_download()
 */
void DwnMainImpl::stop_download()
{
	httpRequestAborted = true;
	http->abort();
	startBtn->setEnabled(true);
	stopBtn->setEnabled(false);
}


/*!
    \fn DwnMainImpl::remove_item()
 */
void DwnMainImpl::remove_item()
{
	QTreeWidgetItem *item = listDownload->currentItem();
	
	if(item)
		delete item;
}


/*!
    \fn DwnMainImpl::slot_new_download()
 */
void DwnMainImpl::slot_new_download()
{
	NewDlgImpl *dlg = new NewDlgImpl(this);
	
	dlg->exec();
	
	if(dlg->b_ok)
	{
		QFile file(dlg->s_file);
		if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
			return;

		QTextStream in(&file);
		while (!in.atEnd()) {
			QString line = in.readLine();
			work_links(line,dlg->s_path);
		}
	}
	
	listDownload->setColumnWidth(0,485);
	listDownload->setColumnWidth(1,185);
	listDownload->setColumnWidth(2,50);
	
	delete dlg;
}



/*!
    \fn DwnMainImpl::work_links(const QString& sfile, const QString& spath)
 */
void DwnMainImpl::work_links(const QString& sfile, const QString& spath)
{
	
    QString stmp = sfile, st;
    
    int i = stmp.indexOf("-");
    
    st = stmp.mid(0,i);
    st = st.simplified(); //Getting link adress
    
    //QMessageBox::about(this,"sfg",st);
    stmp.remove(0,i+1);
    stmp = stmp.simplified();
    
    QString s1 = st.section("/",-2,-2);//Getting directory
    QString si = stmp.section("-",0,0);//Getting start index
    QString sn = stmp.section("-",1,1);//Getting end index
    si=si.simplified();
    sn=sn.simplified();
    s1=s1.simplified();
    
    bool ok;
    int in = si.toInt(&ok);
    if(!ok)
    {
	    error_message();
	    return;
    }
    int n = sn.toInt(&ok);
    if(!ok)
    {
	    error_message();
	    return;
    }
    
    QDir d(spath);
    d.mkdir(s1);
    
    for(int j=in;j<=n;j++)
    {
	    QString s2 = st;
	    
	    s2.replace("(*)",QString::number(j));
	    
	    QTreeWidgetItem *item = new QTreeWidgetItem(listDownload);
	   
	    item->setText(0,s2);
	    item->setText(2,spath+"/"+s1);
	    QProgressBar *b = new QProgressBar(listDownload);
	    listDownload->setItemWidget(item,1,b);
    }
}

bool DwnMainImpl::download_file()
{
	QTreeWidgetItem *item=0;
	
	
   item=listDownload->topLevelItem(0);
   
   if(!item)
	return false; //stop download all files
      
	QUrl url(item->text(0));
	QFileInfo fileInfo(url.path());
	QString fileName = item->text(2);
	fileName += "/"+fileInfo.fileName();
	
	
	if (QFile::exists(fileName)) {
		if (QMessageBox::question(this, tr("HTTP"), 
		    tr("There already exists a file called %1 in "
				    "the current directory. Overwrite?").arg(fileName),
				    QMessageBox::Ok|QMessageBox::Cancel, QMessageBox::Cancel)
				  == QMessageBox::Cancel)
			return true;
		QFile::remove(fileName);
	}
	
	

	file = new QFile(fileName);
	if (!file->open(QIODevice::WriteOnly)) {
		QMessageBox::information(this, tr("HTTP"),
					 tr("Unable to save the file %1: %2.")
							 .arg(fileName).arg(file->errorString()));
		delete file;
		
		file = 0;
		return true;
	}
	
	

	QHttp::ConnectionMode mode = url.scheme().toLower() == "https" ? QHttp::ConnectionModeHttps : QHttp::ConnectionModeHttp;
	http->setHost(url.host(), mode, url.port() == -1 ? 0 : url.port());
    
	if (!url.userName().isEmpty())
		http->setUser(url.userName(), url.password());
		
	

	httpRequestAborted = false;
	httpGetId = http->get(url.path(), file);
	
	//QProgressBar *b = (QProgressBar*)listDownload->itemWidget(item,1); 
	return true;
}

/*!
    \fn DwnMainImpl::start_download()
 */
void DwnMainImpl::start_download()
{
	if(!download_file())
		return;
	
	startBtn->setEnabled(false);
	stopBtn->setEnabled(true);
}

/*!
    \fn DwnMainImpl::slot_exit()
 */
void DwnMainImpl::slot_exit()
{
	stop_download();
	close();
}


/*!
    \fn DwnMainImpl::slot_about()
 */
void DwnMainImpl::slot_about()
{
    QMessageBox::about(this,tr("About DownloadF"),
		       tr("This program download package links from file rules.<br>"
				       "<b>Example file rules:</b><br>"
		"<code>http://www.internetpolyglot.org/ipolyglot_sounds/en/1010/(*)0.mp3 - 11-62</code>"
		"<br>This mask for next lines:<br>"
		"<code>http://www.internetpolyglot.org/ipolyglot_sounds/en/1010/110.mp3<br>"
   		"http://www.internetpolyglot.org/ipolyglot_sounds/en/1010/120.mp3"
		"<p align=\"center\">...</p><br>"
		"http://www.internetpolyglot.org/ipolyglot_sounds/en/1010/620.mp3</code>"));
}

void DwnMainImpl::httpRequestFinished(int requestId, bool error)
{
 //QMessageBox::information(this, "dfdf","slotAuthenticationRequired");
	QTreeWidgetItem *item = listDownload->topLevelItem(0);
	
	if (requestId != httpGetId)
		return;
	if (httpRequestAborted) {
		if (file) {
			file->close();
			file->remove();
			
			delete file;
			file = 0;
		}

		//progressDialog->hide();
		return;
	}

	if (requestId != httpGetId)
		return;

	
	file->close();

	if (error) {
		file->remove();
		QMessageBox::information(this, tr("HTTP"),
					 tr("Download failed: %1.")
							 .arg(http->errorString()));
	}

	delete file;
	
	
	delete item;
	file = 0;
	if(!download_file())
	{
	    startBtn->setEnabled(true);
	    stopBtn->setEnabled(false);
	}
	
}		

void DwnMainImpl::readResponseHeader(const QHttpResponseHeader &responseHeader)
{
//QMessageBox::information(this, "dfdf","readResponseHeader");
	if (responseHeader.statusCode() != 200) {
		QMessageBox::information(this, tr("HTTP"),
					 tr("Download failed: %1.")
							 .arg(responseHeader.reasonPhrase()));
		httpRequestAborted = true;
		
		http->abort();
		return;
	}
}

void DwnMainImpl::updateDataReadProgress(int bytesRead, int totalBytes)
{
//QMessageBox::information(this, "dfdf","updateDataReadProgress");
	if (httpRequestAborted)
		return;

	QTreeWidgetItem *item = listDownload->topLevelItem(0);
	if(!item)
		return;
	QProgressBar *b = (QProgressBar*)listDownload->itemWidget(item,1); 
	b->setMaximum(totalBytes);
	b->setValue(bytesRead);
}


void DwnMainImpl::slotAuthenticationRequired(const QString &hostName, quint16, QAuthenticator *authenticator)
{
   // QMessageBox::information(this, "dfdf","slotAuthenticationRequired");
	QDialog dlg;
	Ui::Dialog ui;
	ui.setupUi(&dlg);
	dlg.adjustSize();
	ui.siteDescription->setText(tr("%1 at %2").arg(authenticator->realm()).arg(hostName));
    
	if (dlg.exec() == QDialog::Accepted) {
		authenticator->setUser(ui.userEdit->text());
		authenticator->setPassword(ui.passwordEdit->text());
	}
}



/*!
    \fn DwnMainImpl::error_message()
 */
void DwnMainImpl::error_message()
{
    QMessageBox::critical(this, tr("Error"),tr("Uncorect data from file"));
}
